%--------------------------------------------------------------------------
%              plotOccam2DMT.m
%
% About:       A Program to plot up OCCAM2DMT inversion models and data.
%
% Usage:       Call this program from the matlab prompt and GUI window pops 
%              up. Their are model plot options in the figure menu, as well
%              as menu items for plotting conductivity profiles and MT
%              responses and data fits.
%
%              Or call as:
%
%              plotOccam2DMT('iterationfilename')   
%
%              where 'iterationfilename' is the name of the iteration to
%              plot. 
%
%              You can also use:
%
%              plotOccam2DMT('lastiter')
%
%              and the last (most recent) iteration will be plotted.      
%
% OPTIONAL PARAMS:
%   'plotcenter', [xmin xmax ymin ymax]       plots only the range given
%   'caxis', [cmin cmax]    color axis values
%   'plotlog', 'off'    uses a linear color scale
%   'x0', n             offsets the x axis by n (uses: x-n
%   'sitectr', 'on'     center the plot on the sites
%   'axes', h           axes to plot in (subplotting) - lots of stuff not done
%                       in this case.
%
%
% Written by:  Kerry Key
%              Scripps Institution of Oceanography
%              kkey@ucsd.edu
%
% Version:     1.0 August 2001
%              1.1 July 2003 - Made a function with callable iteration file 
%              1.2 2006 DGM - various cleanups to keep from crashing.
%              1.3 April, 2007.  Added 'lastiter' option for file name
%              input
%              1.4, October, 2007. All functions are in this single file.
%   
%--------------------------------------------------------------------------

    function plotOccam2DMT(varargin)
    
%--------------------------------------------------------------------------
% Plotting OPTIONS:
%--------------------------------------------------------------------------
    plt_log         = 'on';     % Flag for linear or log resistivity scale:
    plt_cntr        = 'off';    % Flag for plotting just the center:
    plt_caxis       = [];       % color axis to apply
    plt_sitescenter = 'off';    % flag for plotting model as wide as sites
    plot_prej       = 'off';    % do not prejudice model 
    plot_zero       = 0;        % zero point on horizontal axis % kludge for re-zeroing UTM models
    exceptioncolor  = 'w-';     % color of line to plot for model penalty exceptions
    x0 = 0;
    plt_axes = [];
    hfigure = [];
    
    sIterFile = [];
    if nargin==1
        sIterFile = varargin{1};
    elseif nargin>1
        
            
        sIterFile = varargin{1};
        for i=2:2:nargin
            str = varargin{i};
            switch( lower(str) )
                case {'plotcenter'}
                   plt_cntr = 'on';
                   plot_center = varargin{i+1};
                case {'plotlog'}
                   plt_log = varargin{i+1};
                case {'x0'}
                   x0 = varargin{i+1};
                case {'caxis'}
                    plt_caxis = varargin{i+1};
                case {'sitectr'}
                    plt_sitescenter = varargin{i+1};
                case {'axes','axis'}
                    plt_axes = varargin{i+1};
                case {'useprejudicefile'}
                    plot_prej = 'on';
                case {'usefigure'};
                    hfigure = varargin{i+1};
            end
        end
    end

%--------------------------------------------------------------------------
% GET ITERATION FILENAME AND PATH:
%--------------------------------------------------------------------------
    if isempty(sIterFile) || ~exist( sIterFile, 'file' )

% If input file string is  'lastiter', then find last iteration in current
% directory and use that.
        if strcmpi(sIterFile,'lastiter')
            files = dir('*.iter'); % get all iteration files
            
           if isempty(files)
                files = dir('ITER*');
           end
            [temp isort] = sort({files.date});
            isort = fliplr(isort); % descending order
            sIterFile = files(isort(1)).name;            
            [sIterPath,sF,sX] = fileparts(sIterFile);
            sIterFile = [sF sX];
        else % Ask user for file name
            cFileSpec = {'*.iter','Iteration files (*.iter)';'*.*','All Files'};
            [sIterFile,sIterPath] ...
                = uigetfile( cFileSpec, 'Select OCCAM iteration file to plot:' );
            if ~ischar(sIterFile)
                return
            end
       end
    else
        [sIterPath,sF,sX] = fileparts(sIterFile);
        sIterFile = [sF sX];
    end
        
%--------------------------------------------------------------------------
% READ ITERATION FILE:
%--------------------------------------------------------------------------
    [stIter, nParamValues] = readITER( fullfile( sIterPath, sIterFile ) );
    
%--------------------------------------------------------------------------
% READ MODEL FILE:
%--------------------------------------------------------------------------
    [fidmod,modfrm,modnam,moddes,mshfile,mshtyp,stcfile,...
     prjfile,bndofs,nlayer,irz,nrcol,columns,nexep,l1,...
     c1,l2,c2,pen]= readMODEL( fullfile( sIterPath, stIter.sModelFile ) );
 
%--------------------------------------------------------------------------
% READ PREJUDICE FILE
%--------------------------------------------------------------------------
   if strcmp(plot_prej,'on');
        if ~strcmpi(prjfile,'none')
            fid = fopen(prjfile,'r');
            sLine = fgets(fid);
            [sCode, sValue] = strtok( sLine, ':' );
            sCode = lower(strtrim(sCode));
            sValue(1) = [];     % remove the leading token
            sValue = strtrim( strtok(sValue, '!%') );
            if strcmpi(sValue,'OCCAM2MTPREJ_2.0')
                sLine = fgets(fid);
                [sCode, sValue] = strtok( sLine, ':' );
                sCode = lower(strtrim(sCode));
                sValue(1) = [];     % remove the leading token
                sValue = strtrim( strtok(sValue, '!%') );
                nprej = sscanf(sValue,'%i');
                prej = fscanf(fid,'%i %g %g\n',[3 nprej]);
                prej = prej';
            else
                plot_prej='off';
            end
            fclose(fid);
        end
   end
%--------------------------------------------------------------------------
% READ MESH FILE:
%--------------------------------------------------------------------------
    [fidmsh,mshdesc,idx,nodey,nodez,nres,nfre,nexec,...
     fixres,freqs,meshy,meshz,nrc,nrl,mshres]...
     = readMESH( fullfile( sIterPath, mshfile ) );
 
%--------------------------------------------------------------------------
% READ DATA FILE:
%--------------------------------------------------------------------------
    [datfrm,datdes,nsites,sites,offsts,nfre,freqs, ndblks,DATA] ...
        = readDATA( fullfile( sIterPath, stIter.sDataFile ) );
 
%--------------------------------------------------------------------------
% MAKE FE MESH ELEMENTS:
%--------------------------------------------------------------------------
    sumy = cumsum([0; meshy]);
    sumz = cumsum([0; meshz]);
    
    % ADJUST SUMY WITH BINDING OFFSET
    % BNDOFS is the right terminating edge of the left-most reg. brick
    temp = bndofs - sumy(columns(1,1)+1);
    sumy = sumy+temp;
    
    % remove x0 offset for plotting:
    sumy = sumy-x0 - plot_zero;
    sumy = sumy/1000;
    sumz = sumz/1000; % convert to km
    offsts = (offsts - plot_zero) / 1000;
    x0 = x0/1000;
    
    [YY, ZZ, restri] = makeELEMENTS(sumy,sort(-1*sumz),mshres);
    ZZ = -1*ZZ;
    
%--------------------------------------------------------------------------
% MAKE FE MESH LINES:
%--------------------------------------------------------------------------
	[YLy, YLz, ZLy, ZLz] = meshlines(sumy,sumz);
 % h_mesh=plot(YLy,YLz,'k-',ZLy,ZLz,'k-'); axis ij
 % to turn mesh off: set(h_mesh,'visible','off');
 % to trun on mesh:  set(h_mesh,'visible','on');
 
%--------------------------------------------------------------------------
% MAKE REGULARIZATION GRID
%--------------------------------------------------------------------------
    [REGY, REGZ, freebrick, seabrick, fixedbrick] ...
        = assembleREG(nlayer,nrcol,columns,irz,mshres,sumy,sumz);
    
%--------------------------------------------------------------------------
%  MAKE MODEL FIGURE MENUS
%--------------------------------------------------------------------------
    if isempty( plt_axes )
        if isempty(hfigure);
            h_mod = figure;
           % DGM 12/15/2005 - make the figure window wide & put the data descr on it
           % as a window name so we can tell them all apart.
        %   nScrnSize = get(0,'ScreenSize');
            nScrnSize = get(0,'monitorposition'); % Makes figure as big as screen
            if size(nScrnSize,1) > 1 
                % cvt cols 3,4 from right,bottom to width,height
                nScrnSize(:,3) = nScrnSize(:,3) - nScrnSize(:,1) + 1;
                nScrnSize(:,4) = nScrnSize(:,4) - nScrnSize(:,2) + 1;
                % n monitor system, use widest monitor
                nScrnSize = sortrows(nScrnSize, -3); % sort descending by col 3 (width)
                nScrnSize = nScrnSize(1,:);
            end
            nPos = get(h_mod, 'OuterPosition');
            nPos(1) = nScrnSize(1);
            nPos(3) = nScrnSize(3) - 25;	% just a bit smaller.  Matlab doesn't like it when they are just as wide.
            nPos(2) = nPos(2) + nPos(4) - floor( nScrnSize(4) * .9 );
            nPos(4) = floor( nScrnSize(4) * .9 );
            set(h_mod, 'OuterPosition', nPos );
            if ~isempty(datdes)
                set( h_mod, 'Name', [sIterFile ' (' datdes ')'] );
            else
                set( h_mod, 'Name', sIterFile );
            end
            orient landscape
            
        else
            h_mod = hfigure;
            figure(h_mod);
            clf;
        end    

          % Add the figure plotting options menus:

            % Figure pull down menus for showing mesh/reg
            m_main = uimenu( 'Parent', h_mod, 'Label', 'Options' );



            m_mesh = uimenu('Parent',m_main,'Label','FE Mesh');
                mmshonstr = strcat('set(findobj(''tag'',''h_mesh''),''visible'',''on'');',...
                             'set(findobj(''tag'',''mmsh_off''),''checked'',''off'');',...
                             'set(findobj(''tag'',''mmsh_on''),''checked'',''on'')');
                mmsh_on = uimenu('Parent',m_mesh,'Label','On','tag','mmsh_on','Callback',mmshonstr);
                mmshoffstr = strcat('set(findobj(''tag'',''h_mesh''),''visible'',''off'');',...
                              'set(findobj(''tag'',''mmsh_off''),''checked'',''on'');',...
                              'set(findobj(''tag'',''mmsh_on''),''checked'',''off'')');
                mmsh_off= uimenu('Parent',m_mesh,'Label','Off','tag','mmsh_off','Callback',mmshoffstr);
                set(findobj('tag','mmsh_off'),'checked','on');

            m_reg = uimenu('Parent',m_main,'Label','Regularization Grid');
                mreg_onstr =strcat('set(findobj(''tag'',''h_reg''),''edgecolor'',''b'');',...
                                  'set(findobj(''tag'',''mreg_off''),''checked'',''off'');',...
                                  'set(findobj(''tag'',''mreg_on''),''checked'',''on'')');
                mreg_on= uimenu('Parent',m_reg,'tag','mreg_on','Label','Edge On','Callback',mreg_onstr);
                mreg_offstr = strcat('set(findobj(''tag'',''h_reg''),''edgecolor'',''none'');',...
                                    'set(findobj(''tag'',''mreg_off''),''checked'',''on'');',...
                                    'set(findobj(''tag'',''mreg_on''),''checked'',''off'')');
                mreg_off= uimenu('Parent',m_reg,'tag','mreg_off','Label','Edge Off','Callback',mreg_offstr);
                set(mreg_off,'checked','on');   

            % Axis mode menu:
            m_axis = uimenu('Parent',m_main,'Label','Axis');
                maxis_eqstr = strcat('axis equal;',...
                                   'set(findobj(''tag'',''maxis_nm''),''checked'',''off'');',...
                                   'set(findobj(''tag'',''maxis_eq''),''checked'',''on'')');
                maxis_eq=  uimenu('Parent',m_axis,'Label','Equal','tag','maxis_eq','Callback',maxis_eqstr);
                maxis_nmstr = strcat('axis normal;',...
                                   'set(findobj(''tag'',''maxis_nm''),''checked'',''on'');',...
                                   'set(findobj(''tag'',''maxis_eq''),''checked'',''off'')');
                maxis_nm =  uimenu('Parent',m_axis,'Label','Normal','tag','maxis_nm','Callback',maxis_nmstr);
                set(maxis_nm,'checked','on');  
            % Brick Numbering mode menu:
            m_brk = uimenu('Parent',m_main,'Label','Model Brick Numbers');
                mbrk_onstr = strcat('set(findobj(''tag'',''h_brknum''),''visible'',''on'');',...
                                  'set(findobj(''tag'',''mbrk_on''),''checked'',''on'');',...
                                  'set(findobj(''tag'',''mbrk_off''),''checked'',''off'')');
                mbrk_on=  uimenu('Parent',m_brk,'tag','mbrk_on','Label','On','Callback',mbrk_onstr);
                mbrk_offstr = strcat('set(findobj(''tag'',''h_brknum''),''visible'',''off'');',...
                                   'set(findobj(''tag'',''mbrk_off''),''checked'',''on'');',...
                                   'set(findobj(''tag'',''mbrk_on''),''checked'',''off'')');
                mbrk_off =  uimenu('Parent',m_brk,'tag','mbrk_off','Label','Off','Callback',mbrk_offstr);

                set(mbrk_off,'checked','on');    

            % Model Exceptions mode menu:
            m_excep = uimenu('Parent',m_main,'Label','Model Penalty Exceptions');
                mexcep_onstr = strcat('set(findobj(''tag'',''h_excep''),''visible'',''on'');',...
                                  'set(findobj(''tag'',''mexcep_on''),''checked'',''on'');',...
                                  'set(findobj(''tag'',''mexcep_off''),''checked'',''off'')');
                mexcep_on=  uimenu('Parent',m_excep,'tag','mexcep_on','Label','On','Callback',mexcep_onstr);
                mexcep_offstr = strcat('set(findobj(''tag'',''h_excep''),''visible'',''off'');',...
                                   'set(findobj(''tag'',''mexcep_off''),''checked'',''on'');',...
                                   'set(findobj(''tag'',''mexcep_on''),''checked'',''off'')');
                mexcep_off =  uimenu('Parent',m_excep,'tag','mexcep_off','Label','Off','Callback',mexcep_offstr);

                set(mexcep_off,'checked','on');    
   
    end
                 
%--------------------------------------------------------------------------
% PLOT REG GRID:
%--------------------------------------------------------------------------
    if length(freebrick)~=length( nParamValues )
       % KWK March 1, 2007, maybe there are extra parameters for the
       % "static shift" estimation, ignore these:
       if length(freebrick) < length(nParamValues)
            beep;
           disp('Error in plotOccam2DMT: ')
           disp(sprintf('Expected %i bricks, iteration file has: %i',...
               length(freebrick),length(nParamValues)));
           
           disp(sprintf('Ignoring extra model parameter values: \n if these are not static shift parameters, then the model and mesh are incompatible!'))
           
       else
           beep;
           disp('Error in plotOccam2DMT: ')
           disp(sprintf('Expected %i bricks, iteration file has: %i',...
               length(freebrick),length(nParamValues)));
           close gcf;
           edit( fullfile( sIterPath, sIterFile ) );
           return;
       end
    end
    % If ploting prejudice model, make all parameters nans except
    % prejudiced blocks:
    if strcmp(plot_prej,'on')
       nParamValues(:) = nan;
       nParamValues(prej(:,1)) = prej(:,2);
    end

    
    if strcmp(plt_log,'on') % if true, plt log10 resistivity
      h_reg = patch(REGY(:,freebrick),REGZ(:,freebrick),nParamValues(1:length(freebrick))');
    else       
      h_reg = patch(REGY(:,freebrick),REGZ(:,freebrick),10.^nParamValues(1:length(freebrick))');  
    end
    set(h_reg,'tag','h_reg');
    shading flat
    if isempty( plt_axes ) && ~strcmp(plot_prej,'on')   % New plot
        title( sprintf('%s  %s: RMS Misfit = %6.3g,   Roughness =%6.3g' ...
             , datdes, fullfile( sIterPath, sIterFile ), stIter.nMisfit, stIter.nRoughness ) ...
             , 'Interpreter', 'none' );
        if isempty(hfigure) 
            cb = colormap;
            cb = flipud(cb);
            colormap(cb); %flip colormap over so blue is resistive, red conductive
        end
    elseif strcmp(plot_prej,'on')
        title( sprintf('Prejudice Model: %s',prjfile) , 'Interpreter', 'none' )  
          cb = colormap;
        cb = flipud(cb);
        colormap(cb); %flip colormap over so blue is resistive, red conductive
    else
      %  title( sprintf('RMS = %6.3g', stIter.nMisfit ) ...
      %       , 'Interpreter', 'none', 'FontSize', 8 ); 
          title( sprintf('%s  %s: RMS Misfit = %6.3g,   Roughness =%6.3g' ...
             , datdes, fullfile( sIterPath, sIterFile ), stIter.nMisfit, stIter.nRoughness ) ...
             , 'Interpreter', 'none' );
    end
    
    axis ij;
    axis equal;
    hold on;
    if strcmpi(plt_cntr,'on') % plot near surface, model center
        axis equal;
        axis(plot_center)
    end
    if ~isempty(plt_caxis) && length(plt_caxis) == 2
        caxis(plt_caxis);
    else
        % Make sure that we are not less resistive than seawater.
        cax = caxis();
        if strcmpi(plt_log,'on')
            caxis( [max(log10(1/3),cax(1)), cax(2)] );
        else
            caxis( [max(1/3,cax(1)), cax(2)] );
        end
    end
    set(gcf,'renderer','painters');
    % caxis([0 4]);
    if isempty( plt_axes )
        xlabel('Horizontal Position (km)')
        ylabel('Depth (km)');
    end
    box on
    set(gca,'layer','top');
    %   whitebg('k')
%    set(h_reg,'edgecolor','w','linewidth',2);
    set(h_reg,'edgecolor','none','linewidth',2);
    
%--------------------------------------------------------------------------
% NUMBER BRICKS, if:   plt_brknum = 'on'
%--------------------------------------------------------------------------
    if isempty( plt_axes )
        brknum = {1:length(freebrick)};
        h_brknum = text(mean(REGY(:,freebrick)),mean(REGZ(:,freebrick)),brknum,...
                         'fontsize',8,'HorizontalAlignment','center',...
                         'VerticalAlignment','middle');
        set(h_brknum,'visible','off','tag','h_brknum');
    end
    
%--------------------------------------------------------------------------
% Colorbar
%--------------------------------------------------------------------------
   % if isempty( plt_axes )
        cb = colorbar('vert');
        yy = get(cb,'ylabel');
        if strcmpi(plt_log,'on')
            set(yy,'string','Log10 Resistivity (\Omega m)'); 
        else    
            set(yy,'string',' Resistivity (\Omega m)');
        end
    %end
    
%--------------------------------------------------------------------------
% WRITE OVER SEA BRICKS WITH WHITE MESH ELEMENTS.
%--------------------------------------------------------------------------
    Inan = isnan(restri);
    
%--------------------------------------------------------------------------
% PLOT SEA TRIANGLES:
%--------------------------------------------------------------------------
    if ~isempty(Inan)
        seatri(1,1,1:3)=[.7 .7 .7];  % grey triangles
        caxis('manual')
        h_sea=patch(YY(:,Inan),ZZ(:,Inan),seatri,'edgecolor','none');
    end
    
%--------------------------------------------------------------------------
% WRITE OVER AIR BRICKS WITH GRAY MESH ELEMENTS.
%--------------------------------------------------------------------------
    Iair = find(restri==0);
    
%--------------------------------------------------------------------------
% PLOT AIR TRIANGLES:
%--------------------------------------------------------------------------
    if ~isempty(Iair)
        airtri(1,1,1:3)=[.5 .5 .5]; % grey triangles
        caxis('manual')
        h_air=patch(YY(:,Iair),ZZ(:,Iair),airtri,'edgecolor','none');
    end
      
%--------------------------------------------------------------------------
% PLOT FIXED RESISTIVITY MESH ELEMENTS:
%--------------------------------------------------------------------------
    for i = 1:nres
        Ifix = find(restri==i);
        h_fixelmts(i) = patch(YY(:,Ifix),ZZ(:,Ifix),log10(fixres(i)),...
                         'edgecolor','none');
    end
    
%--------------------------------------------------------------------------
% Plot Sites:
%--------------------------------------------------------------------------

    
    % Find nearest node:
    for i = 1:nsites
        [temp,I]  = min(abs(sumy+x0-offsts(i)));
        % I = find(temp==min(temp));       
        ysites(i) = sumy(I);
%        disp( sprintf('%s %8g %8g %i',sites{i}, offsts(i), ysites(i), I) );
        J1 = max(find(mshres(1:4:end,I-1)=='Z'));  %top triangle
        J4 = max(find(mshres(4:4:end,I-1)=='Z'));  %right triangle
        if isempty(J4) % fix for apple mesh right side TP sites
            J4 = 0;
        end
        if isempty(J1)
            J1 = 0;
        end
        if J1>J4       %site is on top of element
             zsites(i) = (sumz(J1));              
        elseif J1==J4  %site on bottom of this element
             zsites(i) = sumz(J1+1);

        elseif J4>J1
            disp('Error in locating site on seafloor');
            disp('There seems to be a seawater cavity at y =')
            disp(offsts(i))
        end
        h_sites(i) = text(ysites(i),zsites(i),'\diamondsuit',...
                     'HorizontalAlignment','center',...
                     'VerticalAlignment','baseline' );
        if isempty( plt_axes )
            set( h_sites(i), 'FontSize',20);
        end
    end
    if strcmpi(plt_sitescenter,'on') % plot near surface, model center
        ax = axis;
        rng = abs(max(ysites)-min(ysites));
        pct = rng * .2;    % show an extra % on each side
        axis( [min(ysites)-pct max(ysites)+pct 0 max(20,rng)]); 
    end
    % PLOT SITE NAMES
    if isempty( plt_axes )
        h_sitesnames = text( ysites, zeros(size(zsites)), sites ...
            , 'VerticalAlignment', 'top' ...
            , 'HorizontalAlignment', 'center' ...
            );
    end
   
%--------------------------------------------------------------------------
% PLOT FE MESH GRID:
%--------------------------------------------------------------------------
    if isempty( plt_axes )
        h_mesh=plot(YLy,YLz,'k-',ZLy,ZLz,'k-');  
        set(h_mesh,'tag','h_mesh');
        set(h_mesh,'visible','off');
    end

    % None of the stuff below this point applies to 2D models that are
    % subplots on a given figure.
    if ~isempty( plt_axes )
        return
    end

%--------------------------------------------------------------------------
% KKey Oct 2006 -- added below
%
% Plot model smoothness penalty exception locations
% note that exception can be any number, but is usually 0
% for letting the model have jumps
%--------------------------------------------------------------------------
    h_excep = [];
    % loop over nexep:
    for i=1:nexep
        % DGM Nov 2006 - may already have brick numbers....
        if isnan( l1(i) )
            br1 = freebrick(c1(i));
            br2 = freebrick(c2(i));
        else
            % get first brick
            br1 = sum(nrcol(1:l1(i)-1))+c1(i);
            % get second brick:
            br2 = sum(nrcol(1:l2(i)-1))+c2(i);
        end
        
        % Find the shared edge.
        % DGM - include diagonals
        %% Vertical penalty
        if REGZ(3,br1)==REGZ(1,br2) && REGY(2,br1)==REGY(2,br2) % br1 above br2
            ypts = [REGY(1,br1) REGY(2,br1)];
            zpts = [REGZ(3,br1) REGZ(3,br1)];
        elseif REGZ(1,br1)==REGZ(3,br2) && REGY(2,br1)==REGY(2,br2) % br2 above br1
            ypts = [REGY(1,br1) REGY(2,br1)];
            zpts = [REGZ(3,br2) REGZ(3,br2)];
            
        %% Horizontal penalty
        elseif REGY(2,br1)==REGY(1,br2) && REGZ(3,br1)==REGZ(3,br2) % br1 left of br2
            ypts = [REGY(2,br1) REGY(2,br1)];
            zpts = [REGZ(2,br1) REGZ(3,br1)];
        elseif REGY(1,br1)==REGY(2,br2) && REGZ(3,br1)==REGZ(3,br2)  % br2 left of br1
            ypts = [REGY(2,br2) REGY(2,br2)];
            zpts = [REGZ(2,br1) REGZ(3,br1)];
            
        %% Diagonal up
        elseif (REGZ(3,br1)==REGZ(1,br2) && REGY(1,br1)==REGY(2,br2)) ... % br1 above right of br2
            || (REGZ(1,br1)==REGZ(3,br2) && REGY(2,br1)==REGY(1,br2)) ... % br2 above right of br1
...        %% Diagonal down
            || (REGZ(3,br1)==REGZ(1,br2) && REGY(2,br1)==REGY(1,br2)) ... % br1 above left of br2
        	|| (REGZ(1,br1)==REGZ(3,br2) && REGY(1,br1)==REGY(2,br2))     % br2 above left of br1
            ypts = [ (REGY(1,br1)+REGY(2,br1))/2 (REGY(1,br2)+REGY(2,br2))/2 ];
            zpts = [ (REGZ(1,br1)+REGZ(3,br1))/2 (REGZ(1,br2)+REGZ(3,br2))/2 ];
        
        end

        % plot the edge:
        h_excep(end+1)= plot(ypts,zpts,exceptioncolor);
        % default is not shown.  Note: use the figure menu to turn penalty plot to
        % visible:
        set(h_excep(end),'linewidth',1,'visible','off','tag','h_excep'); 
    end


%
% Store some model data in the Figure UserData field so that profiles
% can be extracted easily
%

%   Make a REGRHO array with either REG BRICK conductivity
% or else a nan if seawater or air.
    D.REGRHO = nan*ones(size(REGY(1,:)));
    D.REGRHO(freebrick) = nParamValues(1:length(freebrick));
% store REGY,REGZ and REGRHO to the UserData (using structure called D)
    D.REGY= REGY(1:2,:);
    D.REGZ = REGZ([1 3],:);
% Store meshy and meshz for later use:
    D.MESHY = sumy;
    D.MESHZ = sumz;
    set(h_mod,'UserData',D);


%--------------------------------------------------------------------------
% Figure Menu Callbacks To Additional Other Plotting Functions:
%--------------------------------------------------------------------------

    m_ext = uimenu('Parent',h_mod,'Label','Plots');

% Menu to plot another iteration:
    cbstr = sprintf('myaxis=axis(gca);plotOccam2DMT([], ''usefigure'',%g);axis(gca,myaxis);',h_mod);
    m_new = uimenu('Parent',m_ext,'Label','Plot Another Iteration','callback',cbstr);

  % Menu to plot up the data and model responses
    [sIterPath,sF,sX] = fileparts(sIterFile);
    if strcmpi(sF(1:4),'iter')
        respname = sprintf('RESP%2i.resp',stIter.nIteration);
        respname(isspace(respname)) = '0';
    else
        respname = sprintf('%s.resp',sF) ;
    end
    str = sprintf(' plotOccam2DMTResponse(''%s'',''%s'')',respname ...
                 , fullfile( sIterPath, stIter.sDataFile ));
    m_rsp = uimenu('Parent',m_ext,'Label','MT Responses',...
                   'Callback',str);
 
  % Menu item to call plot model misfit and roughness versus iteration
    str = sprintf('plotOccamIterMisfit(''%s'')',fullfile(sIterPath,sIterFile));
    m_msft = uimenu('Parent',m_ext,'Label','Misfit & Roughness',...
               'Callback',str);

  % Menu item to plot data, response and misfit pseudosections for
  % currently plotted 2D model:
    str    = 'Pseudosections';
    cbstr  = sprintf('plotOccam2DMTPseudo(''%s'')',fullfile(sIterPath,sIterFile));   
    m_temp = uimenu('Parent',m_ext,'Label',str, 'Callback',cbstr);             
           
  % Menu item to plot any Prejudice file resistivities in a new figure:
    if ~strcmpi(prjfile,'none')
        str = sprintf('plotOccam2DMT(''%s'',''useprejudicefile'')',fullfile(sIterPath,sIterFile));
        str2 = sprintf('Plot Prejudice Model: %s',prjfile);
        m_temp = uimenu('Parent',m_ext,'Label',str2,'Callback',str);
    end      
           
  % Menu item to call ExtractOCC2DProfile.m function:
    str = sprintf('ExtractOccam2DMTProfile(%g)',h_mod);
    m_prf = uimenu('Parent',m_ext,'Label','Profile',...
               'Callback',str);

  % Menu item to Plot SEO3 olvine-temperature model to the data using QMF:
%     str = sprintf('ApplySEO3_to_OCC2DMTModel(%g)', gcf);
%     m_temp = uimenu('Parent',m_ext,'Label','SEO3 Thermal Model',...
%                'Callback',str);


return

%---------------------------------------
function iorder = getorderbydate(str)
%---------------------------------------
    % sort by date
    % get first and last models   
    num = [];
    iorder=[];
    for i = 1:length(str)
    num(i)  = datenum(str(i).date);
    end
    if ~isempty(num)
        [temp iorder] = sort(num,2,'descend'); 
    end  

return

%--------------------------------------------------------------------------    
function [stIter, nParams] = readITER( sFile )
%--------------------------------------------------------------------------
% readITER - read & parse an Occam2D iteration file.
%
% v1.0, July 2001 by: Kerry Key IGPP/SIO/UCSD
% v2.0, Nov 2006 by David Myer - rewritten to support new OCCAMITER_FLEX
% format files which have new entries for diagonal penalties, and roughness
% modification.  And which no longer require entries to be in a particular
% order.
%
% Parameters:
%   sFile   - path+name of iteration file to open.
% Returns:
%   stIter  - structure with members for each of the header items.  See code
%           for a list of possible members.  Structure will only have members
%           for the items that exist in the iteration file.  So not all
%           members will be present every time.
%   nParams - parameter values
%--------------------------------------------------------------------------

    % Open the iteration file
    fid = fopen( sFile, 'r' );
    
    % Default a value or two that should always exist.
    stIter.nMisfit      = 1000.0;
    stIter.nRoughness   = 0.0;
    
    % The header of the file consists of a bunch of "code: value" pairs
    % terminated by a pair that tells how many parameters there are (and is
    % immediately followed by the parameter list).
    % Read & process the pairs.
    while ~ feof(fid)
        % Get the current line & break up the code / value pair
        sLine = fgets( fid );
        [sCode, sValue] = strtok( sLine, ':' );
        sCode = lower(strtrim(sCode));
        sValue(1) = [];     % remove the leading token
        
        % If there is a user comment in the value, eliminate it.
        sValue = strtrim( strtok(sValue, '!%') );
        
        % Which code do we have?
        switch (sCode)
        case {'format'}
            stIter.sFormat = sValue;
            
        case {'description'}
            stIter.sDesc = sValue;
            
        case {'model file'}
            stIter.sModelFile = sValue;
            
        case {'data file'}
            stIter.sDataFile = sValue;
            
        case {'date/time'}
            stIter.sDateTime = sValue;
            
        case {'max iter', 'iterations to run'}
            stIter.nIterToRun = str2double( sValue );
            
        case {'req tol', 'target misfit'}
            stIter.nTargetMisfit = str2double( sValue );
            
        case {'iruf', 'roughness type'}
            stIter.nRoughnessType = str2double( sValue );
            stIter.bDiagPenalties = (stIter.nRoughnessType < 0);
            stIter.nRoughnessType = abs( stIter.nRoughnessType );
            
        case {'diagonal penalties'}
            stIter.bDiagPenalties = (str2double( sValue ) ~= 0);
            if isfield( stIter, 'nRoughnessType' )
                stIter.nRoughnessType = abs( stIter.nRoughnessType );
            end
            
        case {'debug level'}
            stIter.DebugLevel = str2double( sValue );
            
        case {'iteration'}
            stIter.nIteration = str2double( sValue );
            
        case {'pmu', 'lagrange value'}
            stIter.nLagrange = str2double( sValue );
            
        case {'rlast', 'roughness value'}
            stIter.nRoughness = str2double( sValue );
            
        case {'tlast', 'misfit value'}
            stIter.nMisfit = str2double( sValue );
            
        case {'ifftol', 'misfit reached'}
            stIter.bReachedMisfit = (str2double( sValue ) ~= 0);
            
        case {'modify roughness'}
            stIter.sModifyRoughness = sValue;
            
        case {'no. parms', 'param count'}
            stIter.nParams = str2double( sValue );
            break;	% The next thing in the file is the data param list!
            
            otherwise % do nothing
                
%         case default
%             disp( 'Error reading startup/iteration file!' );
%             error( ['Unknown or unsupported code: ' sCode] );
            
        end
        
    end
    if feof(fid) 
        error( ['Error: iteration file ' sFile ' terminated early!'] );
    end
    
    % Read the parameters
    nParams = fscanf( fid, '%f', stIter.nParams );
    
    % Close up & return
    fclose(fid);

    return




%--------------------------------------------------------------------------
function [fidmod,modfrm,modnam,moddes,mshfile,mshtyp,stcfile,...
         prjfile,bndofs,nlayer,irz,nrcol,columns,nexep,l1,...
         c1,l2,c2,pen]= readMODEL(modfile)
%--------------------------------------------------------------------------
% function readMODEL.m
% Version 1.0, July 2001
% Written by: Kerry Key IGPP/SIO/UCSD
% 
% A matlab script to read an OCCAM2DMT MODEL file
%
% Usage: 
%        [fidmod,modfrm,modnam,moddes,mshfile,mshtyp,stcfile,...
%         prjfile,bndofs,nlayer,irz,nrcol,columns,nexep,l1,...
%         c1,l2,c2,pen]= readMODEL(modfile);
% 
% Inputs
%        modfile: OCCAM2DMT MODEL file name (string format)
%
% Outputs: 
% 		fidmod
% 		modfrm
% 		modnam
% 		moddes
% 		mshfile
% 		mshtyp
% 		stcfile
% 		prjfile
% 		bndofs
% 		nlayer
% 		irz(nlayer)
% 		nrcol(nlayer)
% 		columns(nlayer,nrcol(nlayer))
% 		nexep
% 		l1(nexep)
% 		c1(nexep)
% 		l2(nexep)
% 		c2(nexep)
% 		pen(nexep)
%
%--------------------------------------------------------------------------


% OPEN AND READ IN MODEL FILE:
 
   fidmod  = fopen(modfile,'r');
    cline  = fgets(fidmod);
   modfrm  = sscanf(cline,'FORMAT:           %80c');
    cline  = fgets(fidmod);
   modnam  = sscanf(cline,'MODEL NAME:       %80c');
    cline  = fgets(fidmod);
   moddes  = sscanf(cline,'DESCRIPTION:      %80c');
    cline  = fgets(fidmod);
   mshfile = sscanf(cline,'MESH FILE:        %s');
    cline  = fgets(fidmod);  
   mshtyp  = sscanf(cline,'MESH TYPE:        %s');
    cline  = fgets(fidmod); 
   stcfile = sscanf(cline,'STATICS FILE:     %s');
    cline  = fgets(fidmod);
   prjfile = sscanf(cline,'PREJUDICE FILE:   %s');
    cline  = fgets(fidmod);
   bndofs  = sscanf(cline,'BINDING OFFSET:   %f');
    cline  = fgets(fidmod);
   nlayer  = sscanf(cline,'NUM LAYERS:       %f');
   for i = 1:nlayer
       irz(i)   = fscanf(fidmod,'%i',1);
       nrcol(i) = fscanf(fidmod,'%i',1);
       if i==1;   % initialize columns array  
           columns = nan*ones(nlayer,nrcol(1));
       end
       columns(i,1:nrcol(i)) = (fscanf(fidmod,'%i',nrcol(i)))'; 
   %    disp(sprintf('Nrcol(i): ,length(col),sum(col): %i %i %i\n',nrcol(i),length(find(columns(i,:))),sum(columns(i,1:nrcol(i)))))
   end    
    cline = fgets(fidmod);     %#ok<NASGU> %SUN seems to need this extra line read?
    cline = fgets(fidmod);
   nexep  = sscanf(cline,'%*18c%i');  
   if nexep>0
      for i = 1:nexep
         l1(i)  = fscanf(fidmod,'%i',1);
         c1(i)  = fscanf(fidmod,'%i',1);
         l2(i)  = fscanf(fidmod,'%i',1);
         c2(i)  = fscanf(fidmod,'%i',1);
         pen(i) = fscanf(fidmod,'%f',1);
      end
   elseif nexep < 0     % DGM Nov 2006 - to support KKey's new exception method: blk1, blk2, penalty
        nexep = -nexep;
        for i = 1:nexep
            l1(i) = NaN;
            l2(i) = NaN;
            c1(i) = fscanf( fidmod, '%i', 1 );
            c2(i) = fscanf( fidmod, '%i', 1 );
            pen(i) = fscanf( fidmod, '%f', 1 );
        end
   else %assign l1 l2 c1 c2 pen = nan 
       l1  = nan;
       l2  = nan;
       c1  = nan;
       c2  = nan;
       pen = nan;
   end
   fclose(fidmod);
    return
    
%--------------------------------------------------------------------------
function[fidmsh,mshdesc,idx,nodey,nodez,nres,nfre,nexec,...
         fixres,freqs,meshy,meshz,nrc,nrl,mshres]...
          = readMESH(mshfile)
%--------------------------------------------------------------------------
% function readMESH.m
% Version 1.0, July 2001
% Written by: Kerry Key IGPP/SIO/UCSD
% 
% A matlab script to read an OCCAM2DMT MESH file
%
% Usage: 
%        [fidmsh,mshdesc,idx,nodey,nodez,nres,nfre,nexec,...
%         fixres,freqs,meshy,meshz,nrc,nrl,mshres]...
%          = readMESH(mshfile);
% 
% Inputs
%        mshfile: OCCAM2DMT MESH file name (string format)
%                 ***Can be FWD or INVERSE MESH
%
% Outputs: 
% 			fidmsh
% 			mshdesc
% 			idx
% 			nodey
% 			nodez
% 			nres
% 			nfre
% 			nexec
% 			fixres(nres)
% 			freqs(nfre)
% 			meshy(nodey-1)
% 			meshz(nodez-1)
% 			nrc
% 			nrl(nrc)
% 			mshres(4*(nodeyz-1),nodey-1)
%--------------------------------------------------------------------------

 
% OPEN AND READ IN MESH FILE:   

   fidmsh  = fopen(mshfile,'rt');
   mshdesc = fgets(fidmsh);           % mesh title/description
   idx     = fscanf(fidmsh,'%i',1);   % operation type, 0 to indicate Occam file
   nodey   = fscanf(fidmsh,'%i',1);   % num FE nodes in horizontal direction
   nodez   = fscanf(fidmsh,'%i',1);   % num FE nodes in vertical direction 
   nres    = fscanf(fidmsh,'%i',1);   % num fixed resistivities in mesh, up to 35
   nfre    = fscanf(fidmsh,'%i',1);   % num frequencies (ignored in inversion)
   nexec   = fscanf(fidmsh,'%i',1);   % nexec = 1 for fwd only, =2 for inversion, = 0 to print prameters and mesh
   
   if nres>0 && nres<=35
      fixres = fscanf(fidmsh,'%f',nres);  
   else
      fixres = nan ;
   end
   if nfre>0 
      freqs = fscanf(fidmsh,'%f',nfre); 
   else
      freqs = nan;
   end    
   
   meshy  = fscanf(fidmsh,'%f',nodey-1); % nodey-1 FE column widths, meters
   meshz  = fscanf(fidmsh,'%f',nodez-1); % nodez-1 FE layer heights, meters
   nrc    = fscanf(fidmsh,'%i',1);      % num stations, ignored by OCCAM
   if nrc>0
       nrl = fscanf(fidmsh,'%i',nrc);   %nrc number of nrl node numbers
   else 
       nrl = nan;
   end
   [mshres count] = fscanf(fidmsh,'%1s',[nodey-1 4*(nodez-1) ]);
   mshres = mshres';
   if count~=(nodey-1)*4*(nodez-1) 
       beep
       display('readMESH: Didn''t read correct number of mesh resistivity labels!!!')
   end
   
   
   fclose(fidmsh);

%--------------------------------------------------------------------------
function [YY, ZZ, restri] = makeELEMENTS(sumy,sumz,mshres)
%--------------------------------------------------------------------------    
% makeELEMENTS.m
%
% Written by:  Kerry Key
%              IGPP/SIO/UCSD
% Version:     1.0 August 2001
%
% About:   a function that takes mesh rox and column (y and z) distance vectors 
% along with an array of Wannamaker mesh resistivities and outputs the arrays
% that can plot up the mesh using the matlab intrinsic "patch.m"
% Returns RECTANGULAR ELEMENTS PLUS WITH SEAFLOOR BOUNDARY ELEMENTS AS TRIANGLES.
% Current removes rectanglur elements along seafloor, so no duplicates. 
%
% Inputs: sumy: cumulative sum of y-column increments (+ to right)
%         sumz: cumulative sum of z-column increments (- down)
%         mshres: array of mesh resistivities [4*(sumz-1) by sumy-1]
%                 -4 Rows per each row of sumz for triangles                
%--------------------------------------------------------------------------

   nodey = length(sumy);
   nodez = length(sumz);
% Need 4 coordinates for polygon vertices:
%  MAKE REPEATING IND.
   E1 = repmat([1 2 2 1]',1,nodey-1);
   E2 = repmat(1:nodey-1,4,1);
   Y  = E1+E2-1;
   Y  = repmat(Y,1,nodez-1);
   Z  = repmat([2 2 1 1]',1,(nodez-1))+repmat(0:nodez-2,4,1);
   Z  = repmat (Z,nodey-1,1);
   Z  = reshape(Z,4,(nodez-1)*(nodey-1));
   Z = fliplr(Z);
% NOW ASSIGN sum values:
   YY = sumy(Y);
   ZZ = sumz(Z);
% GET MESH ALPHANUMERICS INTO RES. TABLE INDICIES  (1,9,A-Z,? --> 1:35 + ?)
   temp = double(mshres);                % convert char array to double
   [I]  = find(temp<=57 & temp>= 48);    % mshres is 1-9 characters
      if ~isempty(I);
          temp(I)=temp(I)-48;
      end
   [I]  = find(temp>=65 & temp<=89);     % mshres is A-Y characters
      if ~isempty(I);
         temp(I) = temp(I)-55;
      end
   [Inan]  = find(temp==90);     %make the sea NAN
      if ~isempty(Inan);
         temp(Inan) = 999;
      end
   [I] = find(temp==63);                 % mshres is ?, free parameter
      if ~isempty(I);
          temp(I)= 63;
      end;       %not sure what to do with ? yet.
   lt  = size(temp);
   lt=lt(1);
% FIND TRAINGULAR ELEMENTS:
   [I,J]=find((temp(1:4:lt,:)~=temp(2:4:lt,:) | temp(1:4:lt,:)~=temp(3:4:lt,:) ...
              | temp(1:4:lt,:)~=temp(4:4:lt,:)) & temp(3:4:lt,:)~=999);
          
% reset 999 (sewater resistivities to nan (can't use find w/nan)
     temp(Inan)=nan;
%
     restri = reshape(temp(1:4:lt,:)',1,(nodey-1)*(nodez-1));
% APPEND ON TRIANGULAR ELEMENTS, ADDS 4 NEW COLUMN IND./TRI ELEMENT:
if ~isempty(I) && ~isempty(J)
   TRIEL  = sort(sub2ind(size(mshres'),J,I));
   % GET RID OF SEAFLOOR RECTANGLES, WILL USE TRIANGLES ALONG BOUNDARY
   INDX   = ones(1,length(YY));
   INDX(TRIEL) = 0;
   INDX = logical(INDX);
   mYY    = (YY(1,TRIEL)+YY(2,TRIEL))/2;
   YYT1   = [YY(1,TRIEL)' YY(2,TRIEL)' mYY' YY(1,TRIEL)']';
   YYT4   = [YY(2,TRIEL)' YY(3,TRIEL)' mYY' YY(2,TRIEL)']';
   YYT3   = [YY(3,TRIEL)' YY(4,TRIEL)' mYY' YY(3,TRIEL)']';
   YYT2   = [YY(4,TRIEL)' YY(1,TRIEL)' mYY' YY(4,TRIEL)']';
   YY     = [YY(:,INDX), YYT1, YYT2, YYT3, YYT4];

   mZZ    = (ZZ(1,TRIEL)+ZZ(3,TRIEL))/2;
   ZZT1   = [ZZ(1,TRIEL)' ZZ(2,TRIEL)' mZZ' ZZ(1,TRIEL)']';
   ZZT4   = [ZZ(2,TRIEL)' ZZ(3,TRIEL)' mZZ' ZZ(2,TRIEL)']';
   ZZT3   = [ZZ(3,TRIEL)' ZZ(4,TRIEL)' mZZ' ZZ(3,TRIEL)']';
   ZZT2   = [ZZ(4,TRIEL)' ZZ(1,TRIEL)' mZZ' ZZ(4,TRIEL)']';
   ZZ     = [ZZ(:,INDX), ZZT1, ZZT2, ZZT3, ZZT4];

   I      = (I-1)*4+1;
   TRIEL  = sort(sub2ind(size(mshres'),J,I));
   IND1   = TRIEL;
   IND2   = IND1+nodey-1;
   IND3   = IND1+2*(nodey-1);
   IND4   = IND1+3*(nodey-1);
   temp2  = temp';
 
   restri = [restri(INDX), temp2(IND1)', temp2(IND2)', ...
              temp2(IND3)',temp2(IND4)' ];
end
%--------------------------------------------------------------------------
function [YLy,YLz,ZLy,ZLz ] = meshlines(sumy,sumz)
%--------------------------------------------------------------------------
%    function meshlines.m
%
% Written by:  Kerry Key
%              IGPP/SIO/UCSD
% Version:     1.0 August 2001
%
% About:  makes lines for plotting PW2D MESH grid lines
%
% Usage: [YLy YLz ZLy ZLz ] = meshlines(sumy,sumz)
%
% Plotting the results: h_mesh = plot(YLy,YLz,'k-',ZLy,ZLz,'k-'); 
% Turn mesh off:        set(h_mesh,'visible','off');
% Turn on mesh:         set(h_mesh,'visible','on');
%--------------------------------------------------------------------------
   


% HORIZONTAL LINES:
   lsumy=length(sumy);
   lsumz=length(sumz);
   YLy=[sumy(1).*ones(lsumz,1) ,sumy(lsumy).*ones(lsumz,1)]';
   YLz=[sumz sumz]';
% VERTICAL LINES:
   ZLz=[sumz(1).*ones(lsumy,1) ,sumz(lsumz).*ones(lsumy,1)]';
   ZLy=[sumy sumy]';
 
 % h_mesh=plot(YLy,YLz,'k-',ZLy,ZLz,'k-'); axis ij
 % to turn mesh off: set(h_mesh,'visible','off');
 % to trun on mesh:  set(h_mesh,'visible','on');
 
%-------------------------------------------------------------------------- 
 function [REGY, REGZ, freebrick, seabrick, fixedbrick] = ...
           assembleREG(nlayer,nrcol,columns,irz,mshres,sumy,sumz)
%--------------------------------------------------------------------------
%              function assembleREG.m
%
% Written by:  Kerry Key
%              IGPP/SIO/UCSD
%
% Version:     1.0 August 2001
%
% About:       A function to assemble an OCCAM2DMT regularization grid,. 
%              including consolidated bricks lower down in grid.
%
% Usage:       [REGY, REGZ, freebrick, seabrick, fixedbrick] =
%              assembleREG(nlayer,nrcol,columns,irz,mshres,sumy,sumz);
% 
% Inputs:      nlayer, nrcol, columns, irz, mshres, sumy, sumz 
%
% Outputs:     REGY:       [4 x NBRICKS] array of BRICK Y positions
%              REGZ:       [4 x NBRICKS] array of BRICK Z positions
%              freebrick:  array freebrick indices in REGY and REGZ
%                          (bricks have free inversion parameters)  
%              fixedbrick: array fixedbrick indices in REGY and REGZ 
%                          (bricks are all fixed structure)
%              seabrick:   array seabrick indices in REGY and REGZ 
%                          (bricks are all seawater)
%--------------------------------------------------------------------------

% MAKE POSITION INDICES FOR GENERALIZED REG GRID 
%--------------------------------------------------------------------------
 % FORM REGY: ARRAY OF BRICK CORNER Y LOCATIONS   
   REGY = [];
   for i = 1:nlayer
       sumcols = [1; cumsum(columns(i,1:nrcol(i)))'+1]';
       regy = sumy(sumcols);
       E1 = repmat([1 2 2 1]',1,nrcol(i));
       E2 = repmat(1:nrcol(i),4,1); 
       Y  = E1+E2-1;
       REGY=[REGY regy(Y)];
   end
 % FORM REGZ: ARRAY OF BRICK CORNER Z LOCATIONS   
 % REG. ROW POSTION VECTOR  
   regz=sumz([1 cumsum(irz)+1]);    % distances in z, all columns the same
 % FORM BRICK DEPTH INDICES (1 PER LAYER): 
   Z  = repmat([1 1 2 2]',1,nlayer)+repmat(0:nlayer-1,4,1);
 % NOW REPLICATE nrcol(i) TIMES PER LAYER i
   Z2 = [];
   for i = 1:nlayer                        
      Z2 =[Z2 repmat(Z(:,i),1,nrcol(i))];  
   end
   REGZ=regz(Z2);  % ARRAY OF DEPTHS (m) [4 X #BRICKS]     
 %--------------------------------------------------------------------------
 % SEARCH FOR FREE BRICKS
 %--------------------------------------------------------------------------
 % SEARCH ORDER:  if freebrick, elseif seabrick, elseif fixedbrick

 %  Search bricks starting at top left corner (as res. values are given in 
 %  iteration files.
  seabrick   = [];
  fixedbrick = [];
   inz1    = 1;
   ibrknum = 0;
   freecnt = 1;
   fixcnt  = 1;
   seacnt  = 1;
   for i = 1:nlayer
       inz2 = inz1 + irz(i) - 1;
       iny1 = 1;
% for every brick in that layer 
       for j = 1:nrcol(i)
          ibrknum = ibrknum+1;
% find the right edge of brick
          iny2 = iny1 + abs(columns(i,j)) - 1;
% test to see if the brick is free 
          isfree = 0;
          issea  = 0;
          isfix  = 0;
          for ii = inz1:inz2
             for jj = iny1: iny2 
                if mshres(4*(ii-1)+3,jj) =='?'  %subtriangle #3 is bottom of element 
                   isfree = 1;
                   break;
                elseif mshres(4*(ii-1)+3,jj) =='Z' 
                   issea = 1;
                elseif mshres(4*(ii-1)+3,jj) ~='?' && ...
                       mshres(4*(ii-1)+3,jj) ~='Z'
                   isfix = 1;
                end   
             end
             if isfree == 1
                freebrick(freecnt) = ibrknum;
                freecnt = freecnt + 1;
                break
             end
          end
          if isfree~=1 && issea == 1
             seabrick(seacnt) = ibrknum;
             seacnt = seacnt + 1;
          elseif isfree~=1 && isfix == 1
             fixedbrick(fixcnt) = ibrknum;
             fixcnt = fixcnt + 1; 
          end
          iny1 = iny2 + 1;
       end   % j loop
       inz1 = inz2 + 1;
    end    % i loop
    if isempty(seabrick)
        seabrick = nan;
    end
    if isempty(fixedbrick)
        fixedbrick = nan;
    end    

%--------------------------------------------------------------------------
function [datfrm,datdes,nsites,sites,offsts,nfre,freqs, ...
        ndblks,DATA]= readDATA(datfile)
%--------------------------------------------------------------------------
% function readDATA.m
% Version 1.0, July 2001
% Written by: Kerry Key IGPP/SIO/UCSD
% 
% A matlab script to read an OCCAM2DMT DATA file
%
% Usage: 
%        [datfrm,datdes,nsites,sites,offsts,nfre,freqs, ...
%         ndblks,DATA]= readDATA(datfile);
%
% Inputs
%        datfile: OCCAM2DMT DATA file name (string format)
%
% Outputs: 
% 		datfrm
% 		datdes
% 		nsites
% 		sites(nsites)
% 		offsts(nsites)
% 		nfre:            Also can be found in MESH FILE, if running FWD PW2D 
% 		freqs(nfre):     Also can be found in  MESH FILE, if running FWD PW2D
% 		ndblks:          Number of data blocks
% 		DATA(ndblks,5):  Ndblks by [SITE  FREQ#  TYPE  DATUM  ERR]
%
%--------------------------------------------------------------------------


% OPEN AND READ IN DATA FILE (to get site offsets):
   fiddat = fopen(datfile,'r');
    cline  = fgets(fiddat);  
   datfrm = sscanf(cline,'FORMAT:           %80c');
   if  strcmp(deblank(datfrm),'OCCAM2DMTDATA_2.0')
      ndp = 4;
   else
       ndp =3;
   end
   
    cline  = fgets(fiddat); 
   datdes = sscanf(cline,'DESCRIPTION:      %80c');
    cline  = fgets(fiddat); 
   nsites = sscanf(cline,'SITES:            %f');
   for i  = 1:nsites
       cline    = fgets(fiddat);
      sites(i) = cellstr(sscanf(cline,'%s'));
   end
   string = fgets(fiddat);   %#ok<NASGU>
   offsts = fscanf(fiddat,'%f',nsites);
    cline  = fgets(fiddat);  %#ok<NASGU>
    cline  = fgetl(fiddat); 
   nfre   = sscanf(cline,'FREQUENCIES:%f');
   freqs  = fscanf(fiddat,'%f',nfre);
    cline  = fgets(fiddat); %#ok<NASGU>
    cline  = fgets(fiddat);
   ndblks = sscanf(cline,'DATA BLOCKS:%f');
    string = fgets(fiddat); %#ok<NASGU>
   DATA   = fscanf(fiddat,'%f %f %f %f %f',[ndp+2 ndblks]);
   DATA=DATA';
   fclose(fiddat);
       